
var exports = {};
var _tempVar17 = require("filterClasses");
var Filter = _tempVar17.Filter;
var RegExpFilter = _tempVar17.RegExpFilter;
var BlockingFilter = _tempVar17.BlockingFilter;
var Subscription = require("subscriptionClasses").Subscription;
var defaultMatcher = require("matcher").defaultMatcher;
var FilterNotifier = require("filterNotifier").FilterNotifier;
var Prefs = require("prefs").Prefs;
var _tempVar18 = require("whitelisting");
var checkWhitelisted = _tempVar18.checkWhitelisted;
var getKey = _tempVar18.getKey;
var _tempVar19 = require("url");
var stringifyURL = _tempVar19.stringifyURL;
var extractHostFromFrame = _tempVar19.extractHostFromFrame;
var isThirdParty = _tempVar19.isThirdParty;
var port = require("messaging").port;
var devtools = require("devtools");
var getDecodedHostname = require("url").getDecodedHostname;

var Page = function(id, tab, url)
{
    this.id = id;
    this._tab = tab;
    this._frames = [
                    {
                    url: new URL(url),
                    parent: null
                    }];
    if (tab.page)
    {
        this._messageProxy = new ext._MessageProxy(tab.page);
    }
    else
    {
        this._messageProxy = {
        handleRequest: function()
            {},
        handleResponse: function()
            {},
        sendMessage: function()
            {}
        };
    }
};
Page.prototype = {
    get url()
    {
        return this._frames[0].url;
    }
};

function checkElement(message)
{
    var sender = {}
    sender.page = new Page(0, {}, message.baseURL);
    sender.frame = {url: new URL(message.frame.url), parent: null}
    
    var typeMask = RegExpFilter.typeMap[message.mediatype];
    var documentHost = getDecodedHostname(new URL(message.referrer));
    var sitekey = getKey(sender.page, sender.frame);
    var blocked = false;
    var specificOnly = checkWhitelisted(sender.page, sender.frame, RegExpFilter.typeMap.GENERICBLOCK);
    for (var _loopIndex20 = 0; _loopIndex20 < message.urls.length; ++_loopIndex20)
    {
      var url = message.urls[_loopIndex20];
      var urlObj = new URL(url, message.baseURL);
      var filter = defaultMatcher.matchesAny(stringifyURL(urlObj), typeMask, documentHost, isThirdParty(urlObj, documentHost), sitekey, specificOnly);
      if (filter instanceof BlockingFilter)
      {
        if (filter.collapse != null)
        {
          return filter.collapse;
        }
        blocked = true;
      }
    }
    return blocked && Prefs.hidePlaceholders;
}

function checkPotentialPopup(popupUrl, popupSourceUrl, referrer)
{
  var urlObj = new URL(popupUrl || "about:blank");
  var urlString = stringifyURL(urlObj);
  var documentHost = getDecodedHostname(new URL(referrer));
  var thirdParty = isThirdParty(urlObj, documentHost);

  var sender = {}
  sender.page = new Page(0, {}, referrer);
  sender.frame = {url: new URL(popupSourceUrl), parent: null}

  var specificOnly = !!checkWhitelisted(
    sender.page, sender.frame,
    RegExpFilter.typeMap.GENERICBLOCK
  );

  var filter = defaultMatcher.matchesAny(
    urlString, RegExpFilter.typeMap.POPUP,
    documentHost, thirdParty, null, specificOnly
  );

  return (filter instanceof BlockingFilter)
}

var ElemHide = require("elemHide").ElemHide;
var checkWhitelisted = require("whitelisting").checkWhitelisted;

function getSelectors(message)
{
    var sender = {}
    sender.page = new Page(0, {}, message.baseURL);
    sender.frame = {url: new URL(message.frame.url), parent: null}

  var selectors;

  if (!checkWhitelisted(sender.page, sender.frame,
                        RegExpFilter.typeMap.DOCUMENT |
                        RegExpFilter.typeMap.ELEMHIDE))
  {
    var specificOnly = checkWhitelisted(sender.page, sender.frame,
                                        RegExpFilter.typeMap.GENERICHIDE);
    selectors = ElemHide.getSelectorsForDomain(
      extractHostFromFrame(sender.frame),
      specificOnly ? ElemHide.SPECIFIC_ONLY : ElemHide.ALL_MATCHING
    );
  }
  else
  {
    selectors = [];
  }

  var result = {selectors: selectors, trace: null};
  return JSON.stringify(result);
}

var CSSRules = require("cssRules").CSSRules;

function getCSSFilters(message)
{
    var sender = {}
    sender.page = new Page(0, {}, message.baseURL);
    sender.frame = {url: new URL(message.frame.url), parent: null}

  var filters = [];
  var checkWhitelisted = require("whitelisting").checkWhitelisted;

  if (Prefs.enabled && !checkWhitelisted(sender.page, sender.frame,
                        RegExpFilter.typeMap.DOCUMENT |
                        RegExpFilter.typeMap.ELEMHIDE))
  {
    filters = CSSRules.getRulesForDomain(sender.frame.url.hostname);
    filters = filters.map(function(filter)
    {
      return {
        prefix: filter.selectorPrefix,
        suffix: filter.selectorSuffix,
        regexp: filter.regexpString,
        text: filter.text
      };
    });
  }

  return JSON.stringify(filters);
}
